/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: list.g,v 1.2 2002/02/18 11:28:02 malekith Exp $
 *
 * Standard library for Gont, List module implementation.
 */

open List

int length(<'a>t l)
{
	int length_rec(int res, <'a>t li)
	{
		switch li {
		case Nil[]:
			return res;
		case Cons[hd, tl]:
			return length_rec(res + 1, tl);
		}
	}

	return length_rec(0, l);
}

void iter(*('a) -> void f, <'a>t l)
{
	switch l {
	case Nil[]:
		return;
		
	case Cons[hd, tl]:
		f(hd);
		iter(f, tl);
	}
}

<'b>t map(*('a) -> 'b f, <'a>t l)
{
	switch l {
	case Nil[]:
		return Nil[];		

	case Cons[hd, tl]:
		return Cons[f(hd), map(f, tl)];
	}
}

<'a>t concat(<'a>t x, <'a>t y)
{
	switch x {
	case Nil[]:
		return y;
	case Cons[hd, tl]:
		return Cons[hd, concat(tl, y)];
	}
}

<'a>t filter(*('a) -> bool f, <'a>t l)
{
	switch l {
	case Nil[]:
		return Nil[];
	case Cons[x, xs]:
		if (f(x)) {
			return Cons[x, filter(f, xs)];
		} else {
			return filter(f, xs);
		}
	}
	
}

'a fold_left (*('a, 'b) -> 'a f, 'a init, <'b>t li) 
{
	switch li {
	case Nil[]: 
		return init;
	case Cons[hd, tl]: 
		return fold_left(f, f(init, hd), tl);
	}
}

<'a>t qsort(*('a, 'a) -> int cmp, <'a>t l)
{
	'a m;
	
	switch l {
	case Nil[]:
		return l;
	case Cons[hd, r]:
		m = hd;
		
		<'a>t le = filter(fun ('a x) -> bool is 
						(cmp(x, m) <= 0), r);
		<'a>t gt = filter(fun ('a x) -> bool is 
						(cmp(x, m) > 0), r);
		return concat(qsort(cmp, le), Cons[hd, qsort(cmp, gt)]);
	}
}
